from urllib2 import Request, build_opener
from urllib import urlencode

import httplib

from xml.sax import make_parser, ContentHandler  

from graphtool.tools.common import to_timestamp

url = "http://lxarda16.cern.ch/dashboard/request.py/PAquery"
mandatoryArgs = ('what', 'sortby', 'plot')
validQueries = {'jobs_any' : 'jobs', 'evts_read_any' : 'events read',
                   'time_any' : 'time','time_by_span' : 'batch slots',
                   'CMSSW_fail_codes' : 'job exit code', 
                   'jobs_success_rates' : 'job quality',
                   'event_written_any' : 'events written'}
validSortParams = ('site', 'exit_code', 'prod_team', 'wf', 'dts', 'prod_agent')

validParams = {}
validParams['query'] = {'jobs_any' : 'Jobs', 'evts_read_any' : 'Events read',
                   'time_any' : 'Time','time_by_span' : 'Batch slots',
                   'CMSSW_fail_codes' : 'Exit code', 
                   'jobs_success_rates' : 'Job quality',
                   'event_written_any' : 'Events written'}
validParams['sort'] = {'site' : 'Site', 'prod_team' : 'Production team',
                       'wf' : 'Workflow', 'dts' : 'Dataset', 'exit_code' : 'Exit code',
                       'prod_agent' : 'Production agent'  }

def getPlotData(args):
    """
    Function to connect to Dashboard via http, parse xml 
    and return data in a form compatible with graphtool
    """
    
    results = {}
    
    if not args or not validateOpts(args):
        return results
    
    query_url = formatURL(url, args)
    data = HTTPGetXML(query_url)
    
    try:
        parser = make_parser() 
        handler = plotDataHandler()
        parser.setContentHandler(handler)
        parser.parse(data)
        results = handler.data
        
    except Exception, ex:
        raise "Error parsing data: %s\nData:%s" % (str(ex), str(data)) 
    
    return results


def getWFSummaryData(options):
    """
    get the data from dashboard - act as a proxy
    """
    
    options['what'] = 'wf_summary'
    query_url = formatURL(url, options)
    return HTTPGetXML(query_url)


def HTTPGetXML(query_url):
    """
    connect to url and get xml
    throw exception on error
    """
    try:
        request = Request(query_url)
        request.add_header('Accept', 'text/xml')
        opener = build_opener()
        data = opener.open(request)
            
    except IOError, ex:
        raise "Error contacting Dashboard (See error below):\nurl contacted: %s\n%s" \
                                                            % (query_url, str(ex))
    
    return data
    
    
def formatURL(url, options):
    """
    format a url for the dashboard
    """
    
    query = []
    
    #remove None values
    for opt in options.items():
        if not opt[1] in (None, ''):
            query.append(opt)
    
    return url + '?' + urlencode(query)


def validateOpts(options):
    """
    Check certain mandatory options are present
    """
    for opt in mandatoryArgs:
        if options.get(opt, None) in (None, ''):
            return False
    return True


class plotDataHandler(ContentHandler):
    """
    SAX parser to handle dashboard api plot xml output
    """
    def __init__(self):
        self.isItem = False
        self.isData = False
        self.isName = False
        self.isValue = False
        self.isEpoch = False
        self.isSuccess = False
        self.isFailed = False
        self.isTimeBinned = False
        self.isQualityData = False
        self.name = ''
        self.value = ''
        self.epoch = ''
        self.data = {}
        self.success_jobs = 0
        self.failed_jobs = 0


    def startElement(self, name, attrs):
        if name == 'plot_data':
            self.isData = True
            self.data = {}
        elif name == 'item':
            self.isItem = True
        elif name == 'NAME':
            self.isName = True
        elif name == 'VALUE':
            self.isValue = True
        elif name == 'EPOCH':
            self.isTimeBinned = True
            self.isEpoch = True
        elif name == 'SUCCESS_JOBS':
            self.isQualityData = True
            self.isSuccess = True
        elif name == 'FAILED_JOBS':
            self.isQualityData = True
            self.isFailed = True
        return
    
    
    def characters (self, ch):
        if self.isName:
            self.name = ch
        elif self.isValue:
            self.value = float(ch)
        elif self.isEpoch:
            self.epoch = int(ch)#ch #float(ch)
        elif self.isSuccess:
            self.success_jobs = int(ch)
        elif self.isFailed:
            self.failed_jobs = int(ch)
        #elif self.isItem:
        #    pass
        #elif self.isData:
        #    pass


    def endElement(self, name):
        if name == 'NAME':
            self.isName = False
        elif name == 'VALUE':
            self.isValue = False
        elif name == 'EPOCH':
            #print "end epoch"
            self.isEpoch = False
        elif name == 'SUCCESS_JOBS':
            self.isSuccess = False
        elif name == 'FAILED_JOBS':
            self.isFailed = False
        elif name == 'item' and self.isQualityData and self.isTimeBinned:
            if not self.data.has_key(self.name):
                self.data[self.name] = {}
            #total_jobs = self.success_jobs + self.failed_jobs
            #if (total_jobs > 0):
                #self.data[self.name][self.epoch] = self.success_jobs / total_jobs
            self.data[self.name][self.epoch] = (self.success_jobs, self.failed_jobs)
            self.isItem = False
        elif name == 'item' and self.isTimeBinned and not self.isQualityData:
            if not self.data.has_key(self.name):
                self.data[self.name] = {}
            self.data[self.name][self.epoch] = self.value
            self.isItem = False
        elif name == 'item' and not self.isTimeBinned:
            self.data[self.name] = self.value
            self.isItem = False
        elif name == 'plot_data' and self.isTimeBinned:
            pass
        elif name == 'plot_data' and not self.isTimeBinned:
            pass
            
