from Framework.PluginManager import DeclarePlugin
from Framework import Controller, StaticController, \
                templatepage #\, exposeSerialized, templateXML
from Framework import Logger
from cherrypy import expose
import cherrypy
import ConfigParser
import sys, os, time
from datetime import timedelta

try:
    from graphtool.graphs.common_graphs import *
    from graphtool.web.bonsai import *
    from graphtool.tools.common import to_timestamp, convert_to_datetime
except ImportError, e:
    raise RuntimeError, e
      
from DashboardInterface import getPlotData, getWFSummaryData, validParams


#TODO: Replace **args with list of all args - perhaps could remove __init__
#TODO: Investigate performance
#TODO: Move default values class variables
#TODO: Move graphtool helper functions somewhere
#TODO: Take tooltips from YUI? http://developer.yahoo.com/yui/container/tooltip/
#TODO: format tooltips better
#TODO: Background image to plots?
#TODO: matplotlib config?
#TODO: Check that dashboard query caching is actually working - seem to be 2 queries per plot?
#TODO: Add a list of params and allow changing via plot page ala Phedex page
#TODO: Move graph defaults into graphing class (i.e out of ProdMonPage)
#TODO: sort out class baseUrls and graphtool
#TODO: Handle errors better esp. in dashoard interface


SECS_IN_DAY = 24*60*60
PRODMON_ROOT = os.path.split(__file__)[0] 

class ProdMonPage(Controller):
    """
    Base Class for ProdMon pages
    """
    title = 'Untitled'
    context=''
    logger = None
    baseUrl = ""
    
    def __init__(self, context):
        Controller.__init__ (self, context, __file__)
        self.context = context
        # set top prodmon directory
        self.context.PRODMON_ROOT = PRODMON_ROOT
        # TODO: Find out why this doesnt work
#        self.context.OptionParser().add_option ("--my-prodmon-root",
#                                        help="PRODMON root dir",
#                                        default=os.path.split(__file__)[0],
#                                        dest="prodmon_root")
        self.context.Logger().message("Starting ProdMon")

        #error template page
        self.errorPage = "error"

        #dict for plotting params
        self.plot_defaults = {}
        self.plot_defaults["plot"] = None
        self.plot_defaults["what"] = None
        self.plot_defaults["sortby"] = None
        #self.plot_defaults["starttime"] = None
        #self.plot_defaults["endtime"] = None
        self.plot_defaults["job_type"] = 'Any'
        
        #Graph tool metadata placeholder
        self.metadata = {}
        self.metadata["title"] = "Plot Title"
        #dashboard api returns non cumulative data
        self.metadata["is_cumulative"] = False
        
        #data is returned in time bins depending on range
        # 1hr for a span of < 7 days and 24hr thereafter
        # in tuple to preserve order : format ((max time diff : data binning), (...))
        self.spans = (  (timedelta(days = 7), 3600),
                        (timedelta(days = 31), 86400),
                        (timedelta.max, 86400)
                     )
        
        # Turn off plot cache
        self.use_cache = False

    def getTimeBinning(self, diff):
        """
        Go through self.spans and find first span which covers the given time period
        """
        for timespan, binning in self.spans:
            if timespan > diff:
                return binning
        #if here return default
        return self.spans[-1][1]
    
    
    def updateDefaults(self, options):
        """
        Update user provided dict and return metadata 
                with defaults where appropriate 
        """
        #do we want to urlencode these values?
        
        result = dict(self.plot_defaults)
        now = int(time.time())        
        #result["starttime"] = options.get('starttime', now - 7*SECS_IN_DAY)
        #result["endtime"] = options.get('endtime', now)
        
        #pull in user values
        result.update(options)
        
        #set time based values from this momment if not given

        #reformat dates - use defaults if not specified by user
        #result["starttime"] = convert_to_datetime(result["starttime"])
        #result["endtime"] = convert_to_datetime(result["endtime"])
        result["starttime"] = convert_to_datetime(result.get('starttime', now - 6.9*SECS_IN_DAY))
        result["endtime"] = convert_to_datetime(result.get('endtime', now))
        
        
        #update metadata values from plot values
        metadata = dict(self.metadata)
        metadata['starttime'] = to_timestamp(result["starttime"])
        metadata['endtime'] = to_timestamp(result['endtime'])
        metadata['span'] = self.getTimeBinning(convert_to_datetime(now) \
                                                - result["starttime"])
        
        return result, metadata

    
class ProdMonSummary(ProdMonPage):
    """
    Workflow summary
    """
    
    baseUrl = "/summary"
    
    def __init__(self, context):
        ProdMonPage.__init__(self, context)
        self.apiBaseUrl = "api"
        self.wfApiBaseUrl = "wf"


    @expose
    def api(self, query="", **args):
        """
        proxy the dashboard api - avoid browser security warnings (different hosts)
        """
        
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        
        if query == self.wfApiBaseUrl:
            return getWFSummaryData(args)
        else:
            raise 'unknown "%s" api' % query
    

    #
    # Summary pages 
    #

    @expose
    def workflow(self, **args):
        """
        workflow summary
        """
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'wf_summary'
        vars['plot'] = 'data'
        vars['sortby'] = 'wf'
        return self.templatePage("summary", {"query_url" : "/summary/%s/%s" % (self.apiBaseUrl, self.wfApiBaseUrl),
                                             "query_vars" : vars})


class ProdMonPloter(ProdMonPage, GraphMixIn):
    """
    Class that handles plotting with GraphTool
    """
    graphtypes = {}
    graphtypes['pie'] = {'desc' : 'Pie Charts', 'setter' : None, 'class' : PieGraph, 'graphs' : {}}
    graphtypes['timebargraph'] = {'desc' : 'Bar Graphs', 'setter' : None,  'class' : TimeStackedBarGraph, 'graphs' : {}}
    graphtypes['quality'] = {'desc' : 'Quality Plots', 'setter' : None, 'class' : QualityMap, 'graphs' : {}}
    graphtypes['cumulative'] = {'desc' : 'Cumualtive Plots', 'setter' : None, 'class' : CumulativeGraph, 'graphs' : {}}
    graphtypes['multiple'] = {'desc' : 'Summary Plots', 'setter' : None, 'class' : None, 'graphs' : {}}
    baseUrl = "/plots"


    def __init__(self, context):    
        ProdMonPage.__init__(self, context)
        GraphMixIn.__init__(self)
        self.graphtypes['pie']['setter'] = self.setPieGraph
        self.graphtypes['timebargraph']['setter'] = self.setBarGraph
        self.graphtypes['quality']['setter'] = self.setQualityGraph
        self.graphtypes['cumulative']['setter'] = self.setCumulativeGraph
        self.graphtypes['multiple']['setter'] = self.setMultipleGraph
        self.plotUrl = 'plot'
        self.plotTemplate = "plots"
        
        
    def readyToRun(self):
        """
        run all functions to register all graphs with graphtool
        """
        print "Register graphs with graphtool"
        
        #multiple must be done after constituent graphs
        for type in ('pie', 'timebargraph', 'quality', 'cumulative', 'multiple'):
            graphclass = self.graphtypes[type]['class']
            for name, graph in self.graphtypes[type]['graphs'].items():
                try:
                    graph(self, graphclass, name, {})
                    print ".",
                except:
                    pass        
        print "\nGraphs registered"
    
    
    @expose
    def index(self):
        """
        List of graphs + option to create your own
        """
        return self.templatePage( "plot_index", {"graphs" : self.graphtypes, "plotingUrl" : self.plotUrl})
    
    
    def setBarGraph(self, args):
        args['plot'] = 'bar'
        return self.graphtypes['timebargraph']['class'], self.graphtypes['timebargraph']['graphs']
    
    
    def setPieGraph(self, args):
        args['plot'] = 'pie'
        return self.graphtypes['pie']['class'], self.graphtypes['pie']['graphs']
    
    
    def setQualityGraph(self, args):
        args['plot'] = 'bar'
        return self.graphtypes['quality']['class'], self.graphtypes['quality']['graphs']
    
    
    def setCumulativeGraph(self, args):
        args['plot'] = 'bar'
        return self.graphtypes['cumulative']['class'], self.graphtypes['cumulative']['graphs']
        
    
    def setMultipleGraph(self, args):
        return self.graphtypes['multiple']['class'], self.graphtypes['multiple']['graphs']
        
        
    @expose
    def plot(self, graphtype="freeform", plot="freeform", **args):
        """
        master ploting function, as arguments takes graph type, plot name 
        and a dict of other plot params
        """
        
        #for freeform take graph type from param
        if graphtype == 'freeform':
            graphtype = args.get("plot_type", "pie")
        
        #check graphtype exists
        if not self.graphtypes.has_key(graphtype):
            return self.templatePage(self.errorPage, {"error" : "Graph type '%s' not known" % graphtype})
        
        #get graphtool class and array of relevant plot functions  
        graph, calls = self.graphtypes[graphtype]['setter'](args)
        
        #check plot exists
        if not calls.has_key(plot):
            return self.templatePage(self.errorPage, {"error" : "Plot '%s' unknown" % plot})
            
        #run ploter function for graph
        return calls[plot](self, graph, plot, args)
    
    
    def plotDict(self, plotname, vars, metadata):
        """
        format the dictionary returned to the plot page template
        """
        return {"graph" : self.includeGraph(plotname,  vars),
                    "query_vars" : vars,
                    "title" : metadata['title'],
                    "graphs" : self.graphtypes,
                    "validQueryParams" : validParams}
    
    
    #
    # Plots
    #
    
    
    #
    # Site graphs
    #
    
    
    def evtsReadPerSite(self, graph, plotname, args):
        """
        Events read by site
        """    
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'evts_read_any'
        vars['sortby'] = 'site'
        vars["job_type"] = "Processing"
        metadata["title"]= "Events read per site"
        self.registerGraph( plotname, getPlotData, graph, metadata)        
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['evtsReadPerSite'] = evtsReadPerSite
    graphtypes['timebargraph']['graphs']['evtsReadPerSite'] = evtsReadPerSite
    graphtypes['cumulative']['graphs']['evtsReadPerSite'] = evtsReadPerSite
    
    
    def evtsWrittenPerSite(self, graph, plotname, args):
        """
        Processing events written by site
        """    
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'evts_written_any'
        vars['sortby'] = 'site'
        vars["job_type"] = "Processing"
        metadata["title"]= "Events written per site"
        self.registerGraph( plotname, getPlotData, graph, metadata)        
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['evtsWrittenPerSite'] = evtsWrittenPerSite
    graphtypes['timebargraph']['graphs']['evtsWrittenPerSite'] = evtsWrittenPerSite
    graphtypes['cumulative']['graphs']['evtsWrittenPerSite'] = evtsWrittenPerSite
    
    
    def mergeEvtsWrittenPerSite(self, graph, plotname, args):
        """
        Merge events written per site
        """    
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'evts_written_any'
        vars['sortby'] = 'site'
        vars["job_type"] = "Merge"
        metadata["title"]= "Merge events written"
        self.registerGraph( plotname, getPlotData, graph, metadata)        
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['mergeEvtsWrittenPerSite'] = mergeEvtsWrittenPerSite
    graphtypes['timebargraph']['graphs']['mergeEvtsWrittenPerSite'] = mergeEvtsWrittenPerSite
    graphtypes['cumulative']['graphs']['mergeEvtsWrittenPerSite'] = mergeEvtsWrittenPerSite
    

    def timeSuccessPerSite(self, graph, plotname, args):
        """
        Time spent on successful jobs per site
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "time_success"
        vars["sortby"] = "site"
        #vars["job_type"] = "Any"
        
        metadata["title"] = "Time spent on successful jobs"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['timeSuccessPerSite'] = timeSuccessPerSite
    graphtypes['timebargraph']['graphs']['timeSuccessPerSite'] = timeSuccessPerSite
    graphtypes['cumulative']['graphs']['timeSuccessPerSite'] = timeSuccessPerSite
    
    
    def timeFailedPerSite(self, graph, plotname, args):
        """
        Time spent on successful jobs per site
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "time_failed"
        vars["sortby"] = "site"
        #vars["job_type"] = "Any"
        
        metadata["title"] = "Time spent on successful jobs"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['timeSuccessPerSite'] = timeFailedPerSite
    graphtypes['timebargraph']['graphs']['timeSuccessPerSite'] = timeFailedPerSite
    
    
    def resourcesPerSite(self, graph, plotname, args):
        """
        Approx batch slot usage per production site
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "time_by_span"
        vars["sortby"] = "site"
        #vars["job_type"] = "Any"

        metadata["title"]= "Approx batch slot usage"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['timebargraph']['graphs']['resourcesPerSite'] = resourcesPerSite
    
    
    def qualityPerSite(self, graph, plotname, args):
        """
        Job quality per site
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "jobs_success_rates"
        vars["sortby"] = "site"
        #vars["job_type"] = "Any"
        
        metadata["title"]= "Job quality per site"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['quality']['graphs']['qualityPerSite'] = qualityPerSite
    
    #
    #  Team graphs
    #
    
    
    def evtsReadPerTeam(self, graph, plotname, args):
        """
        Events read by production team
        """    
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'evts_read_any'
        vars['sortby'] = 'prod_team'
        vars["job_type"] = "Processing"
        
        metadata["title"]= "Events read"
        self.registerGraph( plotname, getPlotData, graph, metadata)        
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['evtsReadPerTeam'] = evtsReadPerTeam
    graphtypes['timebargraph']['graphs']['evtsReadPerTeam'] = evtsReadPerTeam
    graphtypes['cumulative']['graphs']['evtsReadPerTeam'] = evtsReadPerTeam
    
    
    def evtsWrittenPerTeam(self, graph, plotname, args):
        """
        Events written by production team
        """    
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'evts_written_any'
        vars['sortby'] = 'prod_team'
        vars["job_type"] = "Processing"
        
        metadata["title"]= "Processing Events Written"
        self.registerGraph( plotname, getPlotData, graph, metadata)        
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['evtsWrittenPerTeam'] = evtsWrittenPerTeam
    graphtypes['timebargraph']['graphs']['evtsWrittenPerTeam'] = evtsWrittenPerTeam
    graphtypes['cumulative']['graphs']['evtsWrittenPerTeam'] = evtsWrittenPerTeam
    graphtypes['cumulative']['graphs']['evtsWrittenPerTeam'] = evtsWrittenPerTeam
    
    
    def mergeEvtsWrittenPerTeam(self, graph, plotname, args):
        """
        Merge events written per team
        """    
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'evts_written_any'
        vars['sortby'] = 'prod_team'
        vars["job_type"] = "Merge"
        metadata["title"]= "Merge events written"
        self.registerGraph( plotname, getPlotData, graph, metadata)        
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['mergeEvtsWrittenPerTeam'] = mergeEvtsWrittenPerTeam
    graphtypes['timebargraph']['graphs']['mergeEvtsWrittenPerTeam'] = mergeEvtsWrittenPerTeam
    graphtypes['cumulative']['graphs']['mergeEvtsWrittenPerTeam'] = mergeEvtsWrittenPerTeam
    
    
    def timeSuccessPerTeam(self, graph, plotname, args):
        """
        Time spent on successful jobs per production team
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "time_success"
        vars["sortby"] = "prod_team"
        #vars["job_type"] = "Any"
        
        metadata["title"] = "Time spent on successful jobs"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['timeSuccessPerTeam'] = timeSuccessPerTeam
    graphtypes['timebargraph']['graphs']['timeSuccessPerTeam'] = timeSuccessPerTeam
    graphtypes['cumulative']['graphs']['timeSuccessPerTeam'] = timeSuccessPerTeam
    
    
    def timeFailedPerTeam(self, graph, plotname, args):
        """
        Time spent on successful jobs per production team
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "time_failed"
        vars["sortby"] = "prod_team"
        #vars["job_type"] = "Any"
        
        metadata["title"] = "Time spent on successful jobs"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['timeSuccessPerTeam'] = timeFailedPerTeam
    graphtypes['timebargraph']['graphs']['timeSuccessPerTeam'] = timeFailedPerTeam
    
    
    def resourcesPerTeam(self, graph, plotname, args):
        """
        Approx batch slot usage per production team
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = """time_by_span"""
        vars["sortby"] = "prod_team"
        #vars["job_type"] = "Any"
        
        metadata["title"]= "Approx batch slot usage"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['timebargraph']['graphs']['resourcesPerTeam'] = resourcesPerTeam


    def qualityPerTeam(self, graph, plotname, args):
        """
        Job quality per team
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "jobs_success_rates"
        vars["sortby"] = "prod_team"
        #vars["job_type"] = "Any"
        
        metadata["title"]= "Job quality per team"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['quality']['graphs']['qualityPerTeam'] = qualityPerTeam
    

    #
    # Workflow graphs
    #

    def evtsReadPerWorkflow(self, graph, plotname, args):
        """
        Events read by workflow
        """    
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'evts_read_any'
        vars['sortby'] = 'wf'
        vars["job_type"] = "Processing"
        
        metadata["title"]= "Events read"
        self.registerGraph( plotname, getPlotData, graph, metadata)        
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['evtsReadPerWorkflow'] = evtsReadPerWorkflow
    graphtypes['timebargraph']['graphs']['evtsReadPerWorkflow'] = evtsReadPerWorkflow
    graphtypes['cumulative']['graphs']['evtsReadPerWorkflow'] = evtsReadPerWorkflow
    
    
    def evtsWrittenPerWorkflow(self, graph, plotname, args):
        """
        Events written by workflow
        """    
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'evts_written_any'
        vars['sortby'] = 'wf'
        vars["job_type"] = "Processing"
        
        metadata["title"]= "Processing Events Written"
        self.registerGraph( plotname, getPlotData, graph, metadata)        
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['evtsWrittenPerWorkflow'] = evtsWrittenPerWorkflow
    graphtypes['timebargraph']['graphs']['evtsWrittenPerWorkflow'] = evtsWrittenPerWorkflow
    graphtypes['cumulative']['graphs']['evtsWrittenPerWorkflow'] = evtsWrittenPerWorkflow    
    
    
    def mergeEvtsWrittenPerWorkflow(self, graph, plotname, args):
        """
        Merge events written per workflow
        """    
        vars, metadata = self.updateDefaults(args)
        vars['what'] = 'evts_written_any'
        vars['sortby'] = 'wf'
        vars["job_type"] = "Merge"
        
        metadata["title"]= "Merge events written"
        self.registerGraph( plotname, getPlotData, graph, metadata)        
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['mergeEvtsWrittenPerWorkflow'] = mergeEvtsWrittenPerWorkflow
    graphtypes['timebargraph']['graphs']['mergeEvtsWrittenPerWorkflow'] = mergeEvtsWrittenPerWorkflow
    graphtypes['cumulative']['graphs']['mergeEvtsWrittenPerWorkflow'] = mergeEvtsWrittenPerWorkflow 
    
    
    def timeSuccessPerWorkflow(self, graph, plotname, args):
        """
        Time spent on successful jobs per workflow
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "time_success"
        vars["sortby"] = "wf"
        #vars["job_type"] = "Any"
        
        metadata["title"] = "Time spent on successful jobs"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['timeSuccessPerWorkflow'] = timeSuccessPerWorkflow
    graphtypes['timebargraph']['graphs']['timeSuccessPerWorkflow'] = timeSuccessPerWorkflow
    graphtypes['cumulative']['graphs']['timeSuccessPerWorkflow'] = timeSuccessPerWorkflow
    
    
    def timeFailedPerWorkflow(self, graph, plotname, args):
        """
        Time spent on successful jobs per workflow
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "time_failed"
        vars["sortby"] = "wf"
        #vars["job_type"] = "Any"
        
        metadata["title"] = "Time spent on successful jobs"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['timeSuccessPerWorkflow'] = timeFailedPerWorkflow
    graphtypes['timebargraph']['graphs']['timeSuccessPerWorkflow'] = timeFailedPerWorkflow
    
    
    def resourcesPerWorkflow(self, graph, plotname, args):
        """
        Approx batch slot usage per workflow
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = """time_by_span"""
        vars["sortby"] = "wf"
        #vars["job_type"] = "Any"
        
        metadata["title"]= "Approx batch slot usage"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['timebargraph']['graphs']['resourcesPerWorkflow'] = resourcesPerWorkflow
    
    
    def qualityPerWorkflow(self, graph, plotname, args):
        """
        Job quality per workflow
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "jobs_success_rates"
        vars["sortby"] = "wf"
        #vars["job_type"] = "Any"
        
        metadata["title"]= "Job quality per workflow"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['quality']['graphs']['qualityPerWorkflow'] = qualityPerWorkflow
    
    
    #
    # Other General plots
    #
    
    def exitCode(self, graph, plotname, args):
        """
        Application exit codes
        """
        vars, metadata = self.updateDefaults(args)
        vars["what"] = "jobs_any" #"CMSSW_fail_codes"
        vars["sortby"] = "exit_code"
        #vars["job_type"] = "Any"
        #vars['exit_code'] = "" #!0
        
        metadata["title"]= "Application exit codes"
        self.registerGraph(plotname, getPlotData, graph, metadata)
        return self.templatePage(self.plotTemplate, self.plotDict(plotname, vars, metadata))
    graphtypes['pie']['graphs']['exitCode'] = exitCode
    graphtypes['timebargraph']['graphs']['exitCode'] = exitCode


    #
    # Free form graph - takes all plot params from url arguments
    #


    def freeform(self, graph, plotname, args):
        """
        takes all settings from params
        """
        vars, metadata = self.updateDefaults(args)
        
        if args.has_key("title"):
            metadata['title'] = args["title"]
        
        self.registerGraph(plotname, getPlotData, graph, metadata)
        dic = self.plotDict(plotname, vars, metadata)
        dic['showPlotChooser'] = True
        return self.templatePage(self.plotTemplate, dic)
    graphtypes['timebargraph']['graphs']['freeform'] = freeform
    graphtypes['pie']['graphs']['freeform'] = freeform
    graphtypes['quality']['graphs']['freeform'] = freeform
    graphtypes['cumulative']['graphs']['freeform'] = freeform
        
       
    #
    # Plots with more than one plot per page
    #
    
    def teamSummary(self, graph, plotname, args):
        """
        Team summary Plots
        """
        vars1, metadata1 = self.updateDefaults(args)
        
        self.setBarGraph(vars1)
        vars1['what'] = 'evts_read_any'
        vars1['sortby'] = 'prod_team'
        vars1["job_type"] = "Processing"
        dict = self.plotDict('evtsReadPerTeam', vars1, metadata1)
        dict['graph1'] = dict.pop('graph')
        
        vars2, metadata2 = self.updateDefaults(args)
        self.setBarGraph(vars2)
        vars2['what'] = 'evts_written_any'
        vars2['sortby'] = 'prod_team'
        vars2["job_type"] = "Processing"
        dict.update(self.plotDict('evtsWrittenPerTeam', vars2, metadata2))
        dict['graph2'] = dict.pop('graph')
        
        vars3, metadata3 = self.updateDefaults(args)
        self.setQualityGraph(vars3)
        vars3["what"] = "jobs_success_rates"
        vars3["sortby"] = "prod_team"
        dict.update(self.plotDict('qualityPerTeam', vars3, metadata3))
        dict['graph3'] = dict.pop('graph')
        
        vars4, metadata4 = self.updateDefaults(args)
        self.setBarGraph(vars4)
        vars4["what"] = """time_by_span"""
        vars4["sortby"] = "prod_team"
        dict.update(self.plotDict('resourcesPerTeam', vars4, metadata4))
        dict['graph4'] = dict.pop('graph')
        
        return self.templatePage("plots", dict)
    graphtypes['multiple']['graphs']['teamSummary'] = teamSummary
    
    
    def siteSummary(self, graph, plotname, args):
        """
        Site summary Plots
        """
        vars1, metadata1 = self.updateDefaults(args)
        
        self.setBarGraph(vars1)
        vars1['what'] = 'evts_read_any'
        vars1['sortby'] = 'site'
        vars1["job_type"] = "Processing"
        dict = self.plotDict('evtsReadPerSite', vars1, metadata1)
        dict['graph1'] = dict.pop('graph')
        
        vars2, metadata2 = self.updateDefaults(args)
        self.setBarGraph(vars2)
        vars2['what'] = 'evts_written_any'
        vars2['sortby'] = 'site'
        vars2["job_type"] = "Processing"
        dict.update(self.plotDict('evtsWrittenPerSite', vars2, metadata2))
        dict['graph2'] = dict.pop('graph')
        
        vars3, metadata3 = self.updateDefaults(args)
        self.setQualityGraph(vars3)
        vars3["what"] = "jobs_success_rates"
        vars3["sortby"] = "site"
        dict.update(self.plotDict('qualityPerSite', vars3, metadata3))
        dict['graph3'] = dict.pop('graph')
        
        vars4, metadata4 = self.updateDefaults(args)
        self.setBarGraph(vars4)
        vars4["what"] = """time_by_span"""
        vars4["sortby"] = "site"
        dict.update(self.plotDict('resourcesPerSite', vars4, metadata3))
        dict['graph4'] = dict.pop('graph')
        
        return self.templatePage("plots", dict)
    graphtypes['multiple']['graphs']['siteSummary'] = siteSummary
    
       
class ProdMonRoot(Controller):
    """
    Framework trick to allow access to js, css directories
    """
    
    def __init__(self, context):      
        Controller.__init__(self, context, __file__)

#map classes to url's
DeclarePlugin ("/Controllers/ProdMon/ProdMonSummary", ProdMonSummary, options={"baseUrl": ProdMonSummary.baseUrl}) 
DeclarePlugin ("/Controllers/ProdMon/ProdMonPloter", ProdMonPloter, options={"baseUrl" : ProdMonPloter.baseUrl})
#Framework trick to allow access to js, css directories
DeclarePlugin ("/Controllers/ProdMon/Root", ProdMonRoot, options={"baseUrl": "/common"})